package com.caliverse.admin.domain.dao.admin;

import java.util.List;
import java.util.Map;

import com.caliverse.admin.domain.entity.InGame;
import com.caliverse.admin.domain.entity.Message;
import com.caliverse.admin.domain.request.NoticeRequest;

public interface NoticeMapper {

    List<InGame> getNoticeList();

    InGame getNoticeDetail(Long id);

    List<Message> getMessage(Long id);

    int postNotice(NoticeRequest noticeRequest);
    int insertMessage(Map map);
    int updateNotice(NoticeRequest noticeRequest);
    int deleteMessage(Long id);
    void deleteNotice(Long id);

    //아이디로 한국어 메시지 가져오기
    String getMessageById(Long id);

    List<InGame> getSchesuleNoticeList();
    int updateStatusNotice(Map map);
    int updateCountNotice(Long id);
}
